package de.uni_bamberg.wiai.kinf.geopacman;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Color;
import android.location.Location;
import android.os.IBinder;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;

import com.esri.android.map.GraphicsLayer;
import com.esri.android.map.MapOptions;
import com.esri.android.map.MapView;
import com.esri.android.map.ags.ArcGISFeatureLayer;
import com.esri.android.map.event.OnStatusChangedListener;
import com.esri.core.geometry.Geometry;
import com.esri.core.geometry.GeometryEngine;
import com.esri.core.geometry.Point;
import com.esri.core.geometry.SpatialReference;
import com.esri.core.io.UserCredentials;
import com.esri.core.map.CallbackListener;
import com.esri.core.map.FeatureSet;
import com.esri.core.map.Graphic;
import com.esri.core.symbol.SimpleLineSymbol;
import com.esri.core.symbol.SimpleMarkerSymbol;
import com.esri.core.tasks.ags.query.Query;
import com.esri.core.tasks.na.NAFeaturesAsFeature;
import com.esri.core.tasks.na.Route;
import com.esri.core.tasks.na.RouteParameters;
import com.esri.core.tasks.na.RouteResult;
import com.esri.core.tasks.na.RouteTask;
import com.esri.core.tasks.query.QueryParameters;

import java.util.List;

public class MapActivity extends AppCompatActivity implements GameService.GameServiceListener {

    private MapView mapView;
    private GraphicsLayer gLayer;
    private boolean mapInitialized = false;

    private int playerGraphicId = -1;

    protected GameService gameService;
    protected boolean gameServiceBound;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_map);

        // Inflate Mapview from XML
        mapView = (MapView) findViewById(R.id.map);

        // Create a graphics layer and add it to the map
        gLayer = new GraphicsLayer();
        mapView.addLayer(gLayer);

        mapView.setMapOptions(new MapOptions(MapOptions.MapType.SATELLITE));
        mapView.setOnStatusChangedListener(new OnStatusChangedListener() {
            @Override
            public void onStatusChanged(Object source, STATUS status) {
                if (OnStatusChangedListener.STATUS.INITIALIZED == status && source == mapView && mapInitialized == false) {
                    mapInitialized = true;

                Log.e("SR", mapView.getSpatialReference().toString());


                String query = "way [highway](39.989929, -0.0726841, 39.9981257, -0.0620537); >; out;";
                List<Node> nodes = OSMWrapperAPI.getNodes(OSMWrapperAPI.getNodesViaOverpass(query));
                for(Node node :nodes) {
                    Point pointGeometry = new Point(node.getLon(), node.getLat());
                    pointGeometry = (Point) GeometryEngine.project(pointGeometry,
                            SpatialReference.create(SpatialReference.WKID_WGS84),
                            mapView.getSpatialReference());
                    SimpleMarkerSymbol simpleMarker = new SimpleMarkerSymbol(Color.YELLOW, 8,
                            SimpleMarkerSymbol.STYLE.CIRCLE);
                    Graphic pointGraphic = new Graphic(pointGeometry, simpleMarker);
                    gLayer.addGraphic(pointGraphic);
                }


                Point pg1 = new Point(-0.0726841, 39.989929);
                pg1 = (Point) GeometryEngine.project(pg1,
                        SpatialReference.create(SpatialReference.WKID_WGS84),
                        mapView.getSpatialReference());
                SimpleMarkerSymbol s1 = new SimpleMarkerSymbol(Color.YELLOW, 8,
                        SimpleMarkerSymbol.STYLE.CIRCLE);
                Graphic from = new Graphic(pg1, s1);

                Point pg2 = new Point(-0.0620537, 39.9981257);
                pg2 = (Point) GeometryEngine.project(pg2,
                        SpatialReference.create(SpatialReference.WKID_WGS84),
                        mapView.getSpatialReference());
                SimpleMarkerSymbol s2 = new SimpleMarkerSymbol(Color.YELLOW, 8,
                        SimpleMarkerSymbol.STYLE.CIRCLE);
                Graphic to = new Graphic(pg2, s2);

                String routeTaskURL = "http://route.arcgis.com/arcgis/rest/services/World/Route/NAServer/Route_World";
                try {
                    UserCredentials creds = new UserCredentials();
                    creds.setUserAccount("geogames_team", "ggt12345");
                    RouteTask routeTask = RouteTask.createOnlineRouteTask(routeTaskURL, creds);
                    RouteParameters routeParams = routeTask.retrieveDefaultRouteTaskParameters();
                    routeParams.setOutSpatialReference(mapView.getSpatialReference());
                    NAFeaturesAsFeature stops = new NAFeaturesAsFeature();
                    stops.setSpatialReference(mapView.getSpatialReference());
                    stops.addFeature(from);
                    stops.addFeature(to);
                    routeParams.setStops(stops);
                    RouteResult mResults = routeTask.solve(routeParams);
                    Route route = mResults.getRoutes().get(0);
                    Log.d("Route", route.toString());
                    Geometry routeGeo = route.getRouteGraphic().getGeometry();
                    gLayer.addGraphic(new Graphic(routeGeo, new SimpleLineSymbol(0x99990055, 5)));
                } catch (Exception e) {
                    Log.e("Route", "Could not create route");
                    e.printStackTrace();
                }
                }
            }
        });

        // Bind to service
        Intent intent = new Intent(this, GameService.class);
        bindService(intent, gameServiceCon, Context.BIND_AUTO_CREATE);
    }

    // ===== Gameplay =====

    @Override
    public void updatePlayerPosition(Location location) {
        if(mapInitialized) {
            if(playerGraphicId != -1) {
                gLayer.removeGraphic(playerGraphicId);
            }
            Point pointGeometry = new Point(location.getLongitude(), location.getLatitude());
            pointGeometry = (Point) GeometryEngine.project(pointGeometry,
                    SpatialReference.create(SpatialReference.WKID_WGS84),
                    mapView.getSpatialReference());
            SimpleMarkerSymbol simpleMarker = new SimpleMarkerSymbol(Color.GREEN, 10,
                    SimpleMarkerSymbol.STYLE.CIRCLE);
            Graphic pointGraphic = new Graphic(pointGeometry, simpleMarker);
            playerGraphicId = gLayer.addGraphic(pointGraphic);
        }
    }

    // ===== Game Service Connection =====

    private ServiceConnection gameServiceCon = new ServiceConnection() {

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            GameService.LocalBinder binder = (GameService.LocalBinder) service;
            gameService = binder.getService();
            gameServiceBound = true;
            gameService.registerListener(MapActivity.this);
        }

        @Override
        public void onServiceDisconnected(ComponentName arg0) {
            gameServiceBound = false;
        }
    };



}

